@@ -1,4 +1,4 @@
-# $Id: XMLNewsNITF.pm,v 1.2 2000/05/06 11:11:58 matt Exp $
+# $Id: XMLNewsNITF.pm,v 1.3 2000/05/19 15:47:13 matt Exp $
 
 package Apache::AxKit::Language::XMLNewsNITF;
 
@@ -14,9 +14,6 @@ sub handler {
 	my $class = shift;
 	my ($r, $xmlfile, $stylefile) = @_;
 	
-	$r->content_type('text/html');
-	$r->content_encoding('utf-8');
-	
 	my $template_processor = XMLNews::HTMLTemplate->new();
 	
 	$template_processor->readTemplate($stylefile);
